@@ -12,6 +12,7 @@ from MAVProxy.modules.mavproxy_map import mp_slipmap
 from MAVProxy.modules.lib import mp_util
 from MAVProxy.modules.lib import mp_settings
 from MAVProxy.modules.lib import mp_module
+from MAVProxy.modules.lib.mp_menu import *
 
 class MapModule(mp_module.MPModule):
     def __init__(self, mpstate):
@@ -25,7 +26,10 @@ class MapModule(mp_module.MPModule):
         self.have_simstate = False
         self.have_vehicle = {}
         self.move_wp = -1
-        self.moving_wp = 0
+        self.moving_wp = None
+        self.moving_fencepoint = None
+        self.moving_rally = None
+        self.mission_list = None
         self.icon_counter = 0
         self.click_position = None
         self.click_time = 0
@@ -49,7 +53,16 @@ class MapModule(mp_module.MPModule):
         self.add_command('map', self.cmd_map, "map control", ['icon',
                                       'set (MAPSETTING)'])
         self.add_completion_function('(MAPSETTING)', self.map_settings.completion)
-        
+
+        self.default_popup = MPMenuSubMenu('Popup', items=[])
+        self.add_menu(MPMenuItem('Fly To', 'Fly To', '# guided ',
+                                 handler=MPMenuCallTextDialog(title='Altitude (m)', default=100)))
+
+    def add_menu(self, menu):
+        '''add to the default popup menu'''
+        self.default_popup.add(menu)
+        self.mpstate.map.add_object(mp_slipmap.SlipDefaultPopup(self.default_popup, combine=True))
+
     def cmd_map(self, args):
         '''map commands'''
         if args[0] == "icon":
@@ -74,14 +87,33 @@ class MapModule(mp_module.MPModule):
     
     def display_waypoints(self):
         '''display the waypoints'''
+        self.mission_list = self.module('wp').wploader.view_list()
         polygons = self.module('wp').wploader.polygon_list()
         self.mpstate.map.add_object(mp_slipmap.SlipClearLayer('Mission'))
         for i in range(len(polygons)):
             p = polygons[i]
             if len(p) > 1:
-                self.mpstate.map.add_object(mp_slipmap.SlipPolygon('mission%u' % i, p,
-                                                              layer='Mission', linewidth=2, colour=(255,255,255)))
-    
+                popup = MPMenuSubMenu('Popup',
+                                      items=[MPMenuItem('Set', returnkey='popupMissionSet'),
+                                             MPMenuItem('WP Remove', returnkey='popupMissionRemove'),
+                                             MPMenuItem('WP Move', returnkey='popupMissionMove')])
+                self.mpstate.map.add_object(mp_slipmap.SlipPolygon('mission %u' % i, p,
+                                                                   layer='Mission', linewidth=2, colour=(255,255,255),
+                                                                   popup_menu=popup))
+
+    def display_fence(self):
+        '''display the fence'''
+        self.fence_change_time = self.module('fence').fenceloader.last_change
+        points = self.module('fence').fenceloader.polygon()
+        self.mpstate.map.add_object(mp_slipmap.SlipClearLayer('Fence'))
+        if len(points) > 1:
+            popup = MPMenuSubMenu('Popup',
+                                  items=[MPMenuItem('FencePoint Remove', returnkey='popupFenceRemove'),
+                                         MPMenuItem('FencePoint Move', returnkey='popupFenceMove')])
+            self.mpstate.map.add_object(mp_slipmap.SlipPolygon('Fence', points, layer=1,
+                                                               linewidth=2, colour=(0,255,0), popup_menu=popup))
+
+            
     def closest_waypoint(self, latlon):
         '''find closest waypoint to a position'''
         (lat, lon) = latlon
@@ -97,16 +129,130 @@ class MapModule(mp_module.MPModule):
             return closest
         else:
             return -1
-            
-    
+
+    def remove_rally(self, key):
+        '''remove a rally point'''
+        a = key.split(' ')
+        if a[0] != 'Rally' or len(a) != 2:
+            print("Bad rally object %s" % key)
+            return
+        i = int(a[1])
+        self.mpstate.functions.process_stdin('rally remove %u' % i)
+
+    def move_rally(self, key):
+        '''move a rally point'''
+        a = key.split(' ')
+        if a[0] != 'Rally' or len(a) != 2:
+            print("Bad rally object %s" % key)
+            return
+        i = int(a[1])
+        self.moving_rally = i
+
+    def selection_index_to_idx(self, key, selection_index):
+        '''return a mission idx from a selection_index'''
+        a = key.split(' ')
+        if a[0] != 'mission' or len(a) != 2:
+            print("Bad mission object %s" % key)
+            return None
+        midx = int(a[1])
+        if midx < 0 or midx >= len(self.mission_list):
+            print("Bad mission index %s" % key)
+            return None
+        mlist = self.mission_list[midx]
+        if selection_index < 0 or selection_index >= len(mlist):
+            print("Bad mission polygon %s" % selection_index)
+            return None
+        idx = mlist[selection_index]
+        return idx
+
+    def move_mission(self, key, selection_index):
+        '''move a mission point'''
+        idx = self.selection_index_to_idx(key, selection_index)
+        self.moving_wp = idx
+        print("Moving wp %u" % idx)        
+
+    def remove_mission(self, key, selection_index):
+        '''remove a mission point'''
+        idx = self.selection_index_to_idx(key, selection_index)
+        self.mpstate.functions.process_stdin('wp remove %u' % idx) 
+
+    def remove_fencepoint(self, key, selection_index):
+        '''remove a fence point'''
+        self.mpstate.functions.process_stdin('fence remove %u' % (selection_index+1)) 
+
+    def move_fencepoint(self, key, selection_index):
+        '''move a fence point'''
+        self.moving_fencepoint = selection_index
+        print("Moving fence point %u" % selection_index) 
+
+    def set_mission(self, key, selection_index):
+        '''set a mission point'''
+        idx = self.selection_index_to_idx(key, selection_index)
+        self.mpstate.functions.process_stdin('wp set %u' % idx) 
+
+    def handle_menu_event(self, obj):
+        '''handle a popup menu event from the map'''
+        menuitem = obj.menuitem
+        if menuitem.returnkey.startswith('# '):
+            cmd = menuitem.returnkey[2:]
+            if menuitem.handler is not None:
+                if menuitem.handler_result is None:
+                    return
+                cmd += menuitem.handler_result
+            self.mpstate.functions.process_stdin(cmd)
+        elif menuitem.returnkey == 'popupRallyRemove':
+            self.remove_rally(obj.selected[0].objkey)
+        elif menuitem.returnkey == 'popupRallyMove':
+            self.move_rally(obj.selected[0].objkey)            
+        elif menuitem.returnkey == 'popupMissionSet':
+            self.set_mission(obj.selected[0].objkey, obj.selected[0].extra_info)
+        elif menuitem.returnkey == 'popupMissionRemove':
+            self.remove_mission(obj.selected[0].objkey, obj.selected[0].extra_info)
+        elif menuitem.returnkey == 'popupMissionMove':
+            self.move_mission(obj.selected[0].objkey, obj.selected[0].extra_info)
+        elif menuitem.returnkey == 'popupFenceRemove':
+            self.remove_fencepoint(obj.selected[0].objkey, obj.selected[0].extra_info)
+        elif menuitem.returnkey == 'popupFenceMove':
+            self.move_fencepoint(obj.selected[0].objkey, obj.selected[0].extra_info)
+
     def map_callback(self, obj):
         '''called when an event happens on the slipmap'''
         from MAVProxy.modules.mavproxy_map import mp_slipmap
+        if isinstance(obj, mp_slipmap.SlipMenuEvent):
+            self.handle_menu_event(obj)
+            return
         if not isinstance(obj, mp_slipmap.SlipMouseEvent):
             return
-        if obj.event.m_leftDown and self.moving_wp != 0:
-            self.moving_wp = 0
-            print("cancelled WP move")
+        if obj.event.m_leftDown and self.moving_rally is not None:
+            self.click_position = obj.latlon
+            self.click_time = time.time()
+            self.mpstate.functions.process_stdin("rally move %u" % self.moving_rally)
+            self.moving_rally = None
+            return
+        if obj.event.m_rightDown and self.moving_rally is not None:
+            print("Cancelled rally move")
+            self.moving_rally = None
+            return
+        if obj.event.m_leftDown and self.moving_wp is not None:
+            self.click_position = obj.latlon
+            self.click_time = time.time()
+            self.mpstate.functions.process_stdin("wp move %u" % self.moving_wp)
+            self.moving_wp = None
+            return
+        if obj.event.m_leftDown and self.moving_fencepoint is not None:
+            self.click_position = obj.latlon
+            self.click_time = time.time()
+            self.mpstate.functions.process_stdin("fence move %u" % (self.moving_fencepoint+1))
+            self.moving_fencepoint = None
+            return
+        if obj.event.m_rightDown and self.moving_wp is not None:
+            print("Cancelled wp move")
+            self.moving_wp = None
+            return
+        if obj.event.m_rightDown and self.moving_fencepoint is not None:
+            print("Cancelled fence move")
+            self.moving_fencepoint = None
+            return
         elif obj.event.m_leftDown:
             if time.time() - self.click_time > 0.1:
                 self.click_position = obj.latlon
@@ -115,35 +261,10 @@ class MapModule(mp_module.MPModule):
         if obj.event.m_rightDown:
             if self.draw_callback is not None:
                 self.drawing_end()
-                return
-            if self.moving_wp == 0:
-                wpnum = self.closest_waypoint(obj.latlon)
-                if wpnum != -1:
-                    self.moving_wp = time.time()
-                    self.move_wp = wpnum
-                    wp = self.module('wp').wploader.wp(self.move_wp)
-                    print("Selected WP %u : %s" % (wpnum, getattr(wp,'comment','')))
-            elif time.time() - self.moving_wp >= 1:
-                wp = self.module('wp').wploader.wp(self.move_wp)
-                (lat, lon) = obj.latlon
-                if getattr(self.console, 'ElevationMap', None) is not None:
-                    alt1 = self.console.ElevationMap.GetElevation(lat, lon) 
-                    alt2 = self.console.ElevationMap.GetElevation(wp.x, wp.y)
-                    wp.z += alt1 - alt2
-                wp.x = lat
-                wp.y = lon
-                
-                wp.target_system    = self.target_system
-                wp.target_component = self.target_component
-                self.moving_wp = 0
-                self.module('wp').loading_waypoints = True
-                self.module('wp').loading_waypoint_lasttime = time.time()
-                self.master.mav.mission_write_partial_list_send(self.target_system,
-                                                                     self.target_component,
-                                                                     self.move_wp, self.move_wp)
-                print("Moved WP %u to %f, %f at %.1fm" % (self.move_wp, lat, lon, wp.z))
-                self.display_waypoints()
-                
+                return                
+            if time.time() - self.click_time > 0.1:
+                self.click_position = obj.latlon
+                self.click_time = time.time()
             
     
     def unload(self):
@@ -178,12 +299,14 @@ class MapModule(mp_module.MPModule):
             return
         self.draw_callback(self.draw_line)
         self.draw_callback = None
+        self.mpstate.map.add_object(mp_slipmap.SlipDefaultPopup(self.default_popup, combine=True))
         self.mpstate.map.add_object(mp_slipmap.SlipClearLayer('Drawing'))
     
     def draw_lines(self, callback):
         '''draw a series of connected lines on the map, calling callback when done'''
         self.draw_callback = callback
         self.draw_line = []
+        self.mpstate.map.add_object(mp_slipmap.SlipDefaultPopup(None))
         
     def mavlink_packet(self, m):
         '''handle an incoming mavlink packet'''
@@ -252,10 +375,7 @@ class MapModule(mp_module.MPModule):
     
         # if the fence has changed, redisplay
         if self.fence_change_time != self.module('fence').fenceloader.last_change:
-            self.fence_change_time = self.module('fence').fenceloader.last_change
-            points = self.module('fence').fenceloader.polygon()
-            if len(points) > 1:
-                self.mpstate.map.add_object(mp_slipmap.SlipPolygon('fence', points, layer=1, linewidth=2, colour=(0,255,0)))
+            self.display_fence()
     
         # if the rallypoints have changed, redisplay
         if self.rally_change_time != self.module('rally').rallyloader.last_change:
@@ -264,8 +384,12 @@ class MapModule(mp_module.MPModule):
             self.mpstate.map.add_object(mp_slipmap.SlipClearLayer('RallyPoints'))
             for i in range(self.module('rally').rallyloader.rally_count()):
                 rp = self.module('rally').rallyloader.rally_point(i)
-                self.mpstate.map.add_object(mp_slipmap.SlipIcon('Rally-%u' % i, (rp.lat*1.0e-7, rp.lng*1.0e-7), icon,
-                                                           layer='RallyPoints', rotation=0, follow=False))
+                popup = MPMenuSubMenu('Popup',
+                                      items=[MPMenuItem('Rally Remove', returnkey='popupRallyRemove'),
+                                             MPMenuItem('Rally Move', returnkey='popupRallyMove')])
+                self.mpstate.map.add_object(mp_slipmap.SlipIcon('Rally %u' % (i+1), (rp.lat*1.0e-7, rp.lng*1.0e-7), icon,
+                                                                layer='RallyPoints', rotation=0, follow=False,
+                                                                popup_menu=popup))
     
         # check for any events from the map
         self.mpstate.map.check_events()
